package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.OSSClient;
import com.example.tdl.common.WarehouseExcel;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.example.tdl.util.AliyunOSSClientUtil;
import com.example.tdl.util.ZipCompressUtil;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.shiro.SecurityUtils;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/download")
public class DownloadController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    Gson gson=new Gson();

    CommFeedback fb=new CommFeedback();

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private InfoRedisService infoRedisService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private I18nController  i18n;

    @Autowired
    private TDLLogService tdlLogService;

    private final static String database ="original";

    @ApiOperation(value = "仓库数据下载",notes = "仓库数据下载")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value = "/download",method = RequestMethod.POST)
    public String downLoad(HttpServletResponse response,HttpServletRequest request,@RequestBody DownloadForWarehouseVo downloadForWarehouseVo) {
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user = gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(downloadForWarehouseVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"warehouseNo"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(downloadForWarehouseVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"TDLSN"));
            return gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(downloadForWarehouseVo.getWarehouseNo()) ==0){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"Nonexistent"));
            return gson.toJson(fb);
        }
        //获取该tdl对应的网关信息
        ResultTDLDeviceVo resultTDLDeviceVo=tdlDeviceService.getByTDLSN(downloadForWarehouseVo.getTDLSN());
        //获取该tdl的报警信息
        ResultAlarmVo resultAlarmVo=tdlLogService.getWarehouseByTdl(downloadForWarehouseVo.getWarehouseNo(),downloadForWarehouseVo.getTDLSN());
        String sqls ="SELECT \"b\", \"T\", \"h\" FROM \"tdl_policy\".\""+resultTDLDeviceVo.getGatewayType()+"_"+resultTDLDeviceVo.getGatewaySN()+"\" WHERE (\"tdl\" = '"+downloadForWarehouseVo.getTDLSN()+"') ORDER BY time DESC LIMIT 1";
        QueryResult queryResults = influxDBTemplate.query(new Query(sqls, database));
        List<DownloadWarehouseVo> downloadWarehouseVos=getWarehouseData(queryResults);
        for (int i=0;i<downloadWarehouseVos.size();i++){
            if (Integer.valueOf(downloadWarehouseVos.get(i).getTemperature()) > resultAlarmVo.getTemMax()||Integer.valueOf(downloadWarehouseVos.get(i).getTemperature())<resultAlarmVo.getTemMin()){
                downloadWarehouseVos.get(i).setAlarm("报警");
            }else{
                downloadWarehouseVos.get(i).setAlarm("正常");
            }
        }
        try {
            String fileName = downloadForWarehouseVo.getWarehouseNo()+"仓库温湿度观测记录表-" +System.currentTimeMillis();
            WarehouseExcel warehouseExcel = new WarehouseExcel(fileName,downloadWarehouseVos,user.getUserName(),response);
            File file=new File("E:"+File.separator+fileName);
            if(!file.exists()){
                file.mkdir();
            }
            HSSFWorkbook wb = warehouseExcel.downloadExcel();
            if(wb !=null){
                request.setCharacterEncoding("utf-8");
                response.setCharacterEncoding("utf-8");
                response.setContentType("application/vnd.ms-excel");
                response.setHeader("Content-Disposition", "attachment; filename="+fileName);
                OutputStream os = new FileOutputStream(new File(file+".xls"));
                wb.write(os);
                os.flush();
                os.close();
                ZipCompressUtil zipCom = new ZipCompressUtil(file+".zip",file+".xls");
                try{
                    zipCom.zip();
                }catch(Exception e){
                    e.printStackTrace();
                    logger.info(e.toString());
                }
                try {
//                    OSSClient ossClient = AliyunOSSClientUtil.getOSSClient();
//                    AliyunOSSClientUtil.uploadObject2OSS(ossClient, new File(file +".zip"));
                    if (new File(file+".xls").delete()){
//                        new File(file+".zip").delete();
                        fb.setCode(1);
                        fb.setMessage("https://tdlcloud.oss-cn-shanghai.aliyuncs.com/warehouse/" +fileName+".zip");
                        return  gson.toJson(fb);
                    }else{
                        fb.setCode(0);
                        fb.setMessage("下载失败");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info(e.toString());
                    fb.setCode(0);
                    fb.setMessage("上传失败");
                }
            }else{
                fb.setCode(0);
                fb.setMessage("上传失败");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("上传失败");
        }
        return  gson.toJson(fb);
    }

    public List<DownloadWarehouseVo> getWarehouseData(QueryResult queryResult){
        List<List<Object>> values = getValues(queryResult);
        List<DownloadWarehouseVo> downloadWarehouseVos = new ArrayList<>();
        for(List<Object> value:values){
            DownloadWarehouseVo downloadWarehouseVo = new DownloadWarehouseVo();
            downloadWarehouseVo.setTime(parseTime(value.get(0).toString()));
            downloadWarehouseVo.setPressure(value.get(1)==null ? "" :value.get(1).toString());
            downloadWarehouseVo.setTemperature(value.get(2)==null ? "" :value.get(2).toString());
            downloadWarehouseVo.setHumidity(value.get(3)==null ? "" :value.get(3).toString());
            downloadWarehouseVos.add(downloadWarehouseVo);
        }
        return downloadWarehouseVos;
    }

    //解析数据格式
    public List<List<Object>> getValues(QueryResult queryResult){
        List<List<Object>> values = new ArrayList<>();
        for(QueryResult.Result result : queryResult.getResults()){
            if(result.getSeries() !=null){
                for (QueryResult.Series series : result.getSeries()) {
                    if(series !=null){
                        values= series.getValues();
                    }
                }
            }
        }
        return values;
    }

    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }

    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }

}
